﻿/*
 *  Copyright 2010 Angus Helm
 * 
 *  This file is part of Terrella.
 *
 *  Terrella is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  Terrella is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with Terrella.  If not, see <http://www.gnu.org/licenses/>.
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Remoting.Channels;

namespace Terrella.WorldMap.AI
{
	/// <summary>
	/// Description of BrainUtils.
	/// </summary>
	public static class BrainUtils
	{
		public static void MoveActor(Actor a, double x, double y) {
			a.X += x;
			a.Y += y;
		}
		
		public static void MoveActorInDirection(Actor a, double direction, double speed) {
			MoveActor(a, speed * Math.Cos(direction), speed * Math.Sin(direction));
		}
		
		public static double DirectionToPoint(double x1, double y1, double x2, double y2) {
			return Math.Atan2(y2 - y1, x2 - x1);
		}
		
		public static double GetFastDistanceBetweenPoints(double x1, double y1, double x2, double y2) {
			return Math.Pow(x1 - x2, 2) + Math.Pow(y1 - y2, 2);
		}
		
		public static double GetDistanceBetweenPoints(double x1, double y1, double x2, double y2) {
			return Math.Sqrt(GetFastDistanceBetweenPoints(x1, y1, x2, y2));
		}
		
		public static double NormalDirection(double dir) {
			while (dir > 2*Math.PI) {
				dir -= 2*Math.PI;
			}
			while (dir < 0) {
				dir += 2*Math.PI;
			}
			return dir;
		}
		
		/**
		 * Returns a list of actors within maxDistance on the world map, and sorted from closest
		 * to furthest away
		 **/
		public static IEnumerable<Actor> GetClosestActors(Actor start, double maxDistance) {
			maxDistance = Math.Pow(maxDistance, 2);
			var actors = from Actor a in start.WorldMap.Actors
				where a != start && GetFastDistanceBetweenPoints(start.X, start.Y, a.X, a.Y) < maxDistance
				select a;
			actors.OrderBy(a => a, new DistanceComparer(start));
			return actors;
		}
		
		public class DistanceComparer : IComparer<Actor> {
			private Actor start;
			public DistanceComparer(Actor a) {
				start = a;
			}
			
			public int Compare(Actor a, Actor b) {
				return ((int)BrainUtils.GetFastDistanceBetweenPoints(start.X, start.Y, a.X, a.Y)) -
					((int)BrainUtils.GetFastDistanceBetweenPoints(start.X, start.Y, b.X, b.Y));
			}
		}
	}
}
